/***************************************************************************************
*
* Block_FIR.ASM
*
* Accompaniment for:
* Guide for Porting Code from SHARC DSPs to TigerSHARC Processors (EE-241)
*
* Written by Andrew Caldwell and Maikel Kokaly-Bannourah 
*	
* June 2004 rev 1.0 
*
* This modified version of the Block FIR code shows a one-to-one conversion (no optimization)
* of the Block FIR SHARC code to TigerSHARC. 
*
***************************************************************************************/


/***********************************************************************

This is an example of a C callable floating point block FIR.
The example is for 64 taps with 128 samples and is provided to show
how the techniques detailed in the accompanying application note can be
applied to this example to convert it for operation on the TigerSHARC
family of processors.

Some of the macros for saving and restoring to and from the stack
have been added simply to show how the TigerSHARC macros provided with
the accompanying application note can be implemented without much
alteration of the code.

************************************************************************/

/*#include  "def21160.h" */ #include "defts101.h"		/* Symbol Definition File */

/* #include "asm_sprt.h" */ #include "asm_sprt_TS.h"

.global _block_fir;

/* program memory code */
/*.section/pm seg_pmco; */	.section program;


_block_fir:
	leaf_entry;

/****************************************************
*	Save various registers to stack
*	This is simply added to show the use of the macros
*****************************************************/	
	/* save all register block to stack */
	save_reg;
	
	/* save some other registers to stack */
/*	r0 = i0;*/				
/*	puts = r0;*/	puts = j0;;

	
/*	r0 = i1;*/
/*	puts = r0;*/	puts = j5;;
	
/*	r0 = i8;*/
/*	puts = r0;*/	puts = k0;;
	
/*	r0 = i9;*/
/*	puts = r0;*/	puts = k5;;

/****************************************************
* 	Retrieve passed arguments
*	The 4 pointers are passed through J7:4
*	The number of samples and number of taps
*	are integer values passed on the stack
*	j4 = input
*	j5 = dline
*	j6 = coeffs
*	j7 = output
*	reads(1) = samples
*	reads(2) = TAPS
*****************************************************/	
	
	/* reads N and TAPS from the stack */
/*	r3 = reads(1);*/	// Output pointer
/*	r2 = reads(2);*/	xr2 = reads(1);	// N number of samples
/*	r1 = reads(3);*/	xr1 = reads(2);	// Number of TAPS 
	
	/* dline pointer CB setup */
	/* second parameter passed stired in r8 */
/*	i0 = r8; b0 = r8; l0 = r1;*/	j0 = j5;; jb0 = j0;; jl0 = xr1;;
	
	/* input samples buffer setup */
	/* first parameter passed stored in r4 */
/*	i1 = r4; b1 = r4; l1 = 0x0;*/	j5 = j4;;
	
	/* coeffs pointer CB setup */
	/* third parameter passed stored in r12 */
/*	i8 = r12; b8 = r12;	l8 = r1;*/	k0 = j6;; kb0 = k0;; kl0 = xr1;;
	
	/* output buffer setup */
/*	i9 = r3; b9 = r3; l9 = 0x0;*/	k5 = j7;;
	
	/* Setup modify registers for arrays */ 
/*	m0 = 0;*/	j12 = 0;;
/*	m1 = 1;*/	j13 = 1;;
/*	m2 = -1;*/	j14 = -1;;
/*	m3 = 2;*/	j15 = 2;;
/* 	m9 = 2;*/	k13 = 2;;
/*	m10= 1;*/	k14 = 1;;
	
	/* Zero the first output result */
/*	f8 = 0.0;*/	xr8 = 0;;
	
	/* Circular Buffer Enable */
/*	bit set MODE1 CBUFEN;*/
	
	/* Circular Buffering not in effect until next cycle */				
/*	nop;*/
	
	/* move pointer to delay[1] */
/*	s0 = dm(i0, m1);*/ yr0 = CB [j0+=j13];;
	
	/* load s0 with the value of delay[1] for SIMD store, move pointer to delay[0] */					
/*	s0 = dm(i0, m2);*/	yr0 = CB [j0+=j14];;

	/* r1 = taps/2 due to SIMD mode */
/*	r1 = lshift r1 by -1;*/	xr1 = lshift r1 by -1;;
	
	/* 3 macs outside of fir mac loop */				
/*	r3 = 3;*/	xr3 = 3;;
	
	/* r3 = taps/2 - 3 for fir mac loop counter */ 							
/*	r3 = r1 - r3;*/	xr3 = r1 - r3;;

	/* SIMD Mode Enable */
/*    bit set MODE1 PEYEN;*/
    
    /* SIMD not in effect until next cycle */
    /* read one sample from INPUT[i] */
/*  r0 = dm(i1,m1);*/	xr0 = [j5+=j13];;
    
    /* read 2 coeffs */										
/* 	f4 = pm(i8,m9);*/	yxr4 = CB l[k0+=k13];;
   	
   	/* outer loop - sample loop */					
/*	lcntr = r2, do main_fir until lce;*/ LC0 = xr2;;

/********************************************************************************************************
* In the following section of code the delay line is not always aligned on a dual address so we are 
* unable to perform SIMD accesses on the TigerSHARC. There are many ways to overcome this problem, 
* such as copying the delay line to another memory block then using a J and K pointer to load two 
* words in a single instruction. Another method would be to implement quad loads with the use of the
* data alignment buffer on TigerSHARC. In this original conversion we simply perform two loads instead
* of a SIMD load for the delay line with each having a modify of 1 instead of 2. A SIMD load however is
* performed for all coefficient accesses as the pointer in this buffer is always aligned.
********************************************************************************************************/		
		/* transfer new sample and last new sample to delayline */
main_fir:		/*dm(i0,m3)=f0;*/	CB [j0+=j13] = xr0;;
									CB [j0+=j13] = yr0;;
		
		/* samples * coeffs, read 2 samples, read 2 coeffs */			
/*		f8=f0*f4, f0=dm(i0,m3), f4=pm(i8,m9);*/	fr8 = r0*r4; xr0 = CB [j0+=j13]; yxr4 = CB l[k0+=k13];;
												yr0 = CB [j0+=j13];;
		
		/* samples * coeffs, read 2 samples, read 2 coeffs */	
/*		f12=f0*f4, f0=dm(i0,m3), f4=pm(i8,m9);*/	fr12 = r0*r4; xr0 = CB [j0+=j13]; yxr4 = CB l[k0+=k13];;
													yr0 = CB [j0+=j13];;
  
		/* FIR loop */
/*		lcntr=r3, do macs until lce;*/ LC1 = xr3;;

		/* samples * coeffs, accum, read 2 samples, read 2 coeffs */			
macs:   /*f12=f0*f4, f8=f8+f12, f0=dm(i0,m3), f4=pm(i8,m9);*/ 	fr12 = r0*r4; fr8 = r8+r12; xr0 = CB [j0+=j13]; yxr4 = CB l[k0+=k13];;
																if NLC1E, jump macs; yr0 = CB [j0+=j13];; 	
        
		/* samples * coeffs, accum, load s0 with the value of delay[1] for SIMD store, move pointer to delay[0] */
/*		f12=f0*f4, f8=f8+f12, s0=dm(i0,m2);*/	fr12 = r0*r4; fr8 = r8+r12; yr0 = CB [j0+=j14];;
        
		/* final SIMD accum, read new sample into s10 */
/*		f8=f8+f12, s10=dm(i1,m1);*/	fr8 = r8+r12; yr10 = [j5+=j13];;
        
		/* move PEy total into PEx register file */
/*		r12=s8;*/	xr12 = yr8;;
		
		/* last accum, read sample into s0 for first MAC of next sample, read 2 coeffs */
/*		f8=f8+f12, f4=pm(i8,m9);*/	fr8 = r8+r12; yxr4 = CB l[k0+=k13];;

		/* place new sample from s10 into f0 without corrupting s0, swaps f0 and s10 only */
/*		f0 <-> s10;*/	xr0 = yr10;;
		
		/* write result to OUTPUT[i] */
/*main_fir:*/	/*pm(i9,m10)=f8;*/if NLC0E, jump main_fir; [k5+=k14] = xr8;;

	/* Circular Buffer Disable, SIMD Mode Disable */
/*	bit clr MODE1 CBUFEN | PEYEN;*/
	
	/* restore registers from stack */
/*	r0 = gets(1);*/
/*	i9 = r0;*/		k5 = gets(1);
	
/*	r0 = gets(2);*/
/*	i8 = r0;*/		k0 = gets(2);
	
/*	r0 = gets(3);*/
/*	i1 = r0;*/		j5 = gets(3);
	
/*	r0 = gets(4);*/
/*	i0 = r0;*/		j0 = gets(4);
	
	/* modify stack pointer */
/*	alter(4);*/ alter(4);
	
	/* restore register block from stack */
	restore_reg;
		
_block_fir.END:	leaf_exit;
